<?php

namespace Drupal\cloudfront_edge_caching\Controller;

use Drupal\cloudfront_edge_caching\CloudfrontEdgeCache;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Cloudfront Edge Cache Controller.
 */
class CloudfrontEdgeCacheController extends ControllerBase {

  /**
   * The cloudfront edge cache service.
   *
   * @var \Drupal\cloudfront_edge_caching\CloudfrontEdgeCache
   */
  protected $cloudfrontEdgeCache;

  /**
   * Constructor.
   *
   * @param \Drupal\cloudfront_edge_caching\CloudfrontEdgeCache $cloudfront_edge_cache
   *   The cloudfront edge cache service.
   */
  public function __construct(CloudfrontEdgeCache $cloudfront_edge_cache) {
    $this->cloudfrontEdgeCache = $cloudfront_edge_cache;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('cloudfront_edge_cache'),
    );
  }

  /**
   * Get distributions.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return array
   *   Return table of distributions.
   */
  public function getDistributions(Request $request): array {
    $build = [
      '#type' => 'table',
      '#empty' => $this->t('No distributions available at this time.'),
    ];

    $build['#header'] = [
      'id' => $this->t('Distribution Id'),
      'status' => $this->t('Status'),
      'modified' => $this->t('Last Modified'),
    ];

    $rows = [];
    $distributions = $this->cloudfrontEdgeCache->getDistributions();
    if (isset($distributions['DistributionList'])) {
      if ($distributions['DistributionList']['Quantity'] > 0) {
        foreach ($distributions['DistributionList']['Items'] as $distribution) {
          /** @var \Aws\Api\DateTimeResult $created */
          $modified = $distribution['LastModifiedTime'];
          $rows[] = [
            'id' => $distribution['Id'],
            'status' => $distribution['Status'],
            'modified' => ($modified) ? $modified->format('Y-m-d H:i:s') : '',
          ];
        }
      }
    }
    $build['#rows'] = $rows;

    return $build;
  }

  /**
   * Get invalidations.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return array
   *   Return table of invalidations.
   */
  public function getInvalidations(Request $request): array {
    $build = [
      '#type' => 'table',
      '#empty' => $this->t('No invalidations available at this time.'),
    ];

    $build['#header'] = [
      'id' => $this->t('Invalidation Id'),
      'status' => $this->t('Status'),
      'created' => $this->t('Created'),
    ];

    $rows = [];
    $invalidations = $this->cloudfrontEdgeCache->getInvalidations();
    if (isset($invalidations['InvalidationList'])) {
      if ($invalidations['InvalidationList']['Quantity'] > 0) {
        foreach ($invalidations['InvalidationList']['Items'] as $invalidation) {
          /** @var \Aws\Api\DateTimeResult $created */
          $created = $invalidation['CreateTime'];
          $rows[] = [
            'id' => $invalidation['Id'],
            'status' => $invalidation['Status'],
            'created' => ($created) ? $created->format('Y-m-d H:i:s') : '',
          ];
        }
      }
    }
    $build['#rows'] = $rows;

    return $build;
  }

}
